# -*- coding: utf-8 -*-
###############################################################################
#   Copyright (c) 2019 Eynes S.R.L (Santiago Said)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
###############################################################################

import logging
import datetime

from odoo import models, fields, api, _
from odoo.exceptions import ValidationError, UserError

_logger = logging.getLogger(__name__)


class MedicalBenefitDescriptorType(models.Model):
    _name = "medical.benefit.descriptor.type"
    _description = "Medical Benefit Descriptor Type"

    name = fields.Char(required=True)
    code = fields.Char(required=True)

class MedicalBenefitDescriptorValidator(models.Model):
    _name = "medical.benefit.descriptor.validator"
    _description = "Medical Benefit Descriptor Validator"

    name = fields.Char('Name', compute='_compute_name')
    period = fields.Selection(
            [
                ('1', 'Per day'),
                ('7', 'Per week'),
                ('30', 'Per month'),
                ('365', 'Per year'),
                ('0', 'Per pacient'),
            ],
            string='period',
            default='1',
    )
    control = fields.Selection(
            [
                ('per_eye', 'Per eye'),
                ('total', 'In total'),
            ],
            string='control',
            default='per_eye',
            size='4',
    )

    quantity = fields.Float('Quantity of applications')

    @api.depends('quantity','period','control')
    @api.multi
    def _compute_name(self):
        for vr in self:
            control = dict(vr.fields_get(['control'])['control']['selection'])[vr.control]
            period = dict(vr.fields_get(['period'])['period']['selection'])[vr.period]
            vr.name = "%s - %s - %s" % (vr.quantity, _(control), _(period))

    @api.multi
    def name_get(self):
        res = []
        for vr in self:
            control = dict(vr.fields_get(['control'])['control']['selection'])[vr.control]
            period = dict(vr.fields_get(['period'])['period']['selection'])[vr.period]
            res.append((vr.id,"%s - %s - %s" % (vr.quantity, _(control), _(period))))
        return res


class MedicalBenefitDescriptor(models.Model):
    _name = "medical.benefit.descriptor"
    _description = "Medical Benefit Descriptor"

    name = fields.Char(string='Internal Code', required=True, index=True)
    name_os = fields.Char(string='Health Insurance Code', required=True)
    description = fields.Char(string='Description', required=True)
    type_id = fields.Many2one('medical.benefit.descriptor.type', string='Type')
    short_desc = fields.Char(compute='_compute_short_desc')
    period = fields.Date()
    amount_galeno = fields.Float('Amount expressed in galenos')
    requires_eye = fields.Boolean('Requires eye specification', default=True);
    validator_id = fields.Many2one('medical.benefit.descriptor.validator', string='Validator')
    is_translation = fields.Boolean('Is translation?')
    health_insurance_id = fields.Many2one('res.partner', string='Health Insurance', default=lambda x: x.get_default_health_insurance_id())
    translated_descriptor_id = fields.Many2one('medical.benefit.descriptor',string='SOFSA descriptor')
    eye_id = fields.Many2one('medical.benefit.eye','Eye')

    @api.multi
    def get_default_health_insurance_id(self):
        insurance = self.env.ref('health_insurance.res_partner_sofsa')
        return insurance

    @api.model
    def check_code(self):
        for ds in self:
            descriptor = self.env['medical.benefit.descriptor'].search([
                ('id','!=',ds.id),
                ('name','=',ds.name),
                ('is_translation','=',ds.is_translation),
                ('health_insurance_id','=',ds.health_insurance_id.id)
            ])
            if descriptor:
                raise ValidationError(_("Code %s already exists!") % (ds.name))


    @api.model
    def create(self, vals):
        res = super(MedicalBenefitDescriptor, self).create(vals)
        if not res.validator_id:
            res._onchange_type_id()
        res.check_code()
        if not res.is_translation:
            res.health_insurance_id = self.env.ref('health_insurance.res_partner_sofsa')
        return res

    @api.onchange('is_translation')
    def onchange_is_translation(self):
        if self.is_translation:
            self.health_insurance_id = False
        else:
            self.health_insurance_id = self.env.ref('health_insurance.res_partner_sofsa')

    @api.multi
    def write(self, vals):
        super(MedicalBenefitDescriptor, self).write(vals)
        self.check_code()

    @api.onchange('type_id')
    def _onchange_type_id(self):
        if self.type_id.id == self.env.ref('medical_benefit.mbdt_cons').id:
            self.validator_id = self.env.ref('medical_benefit.mbdv_cons')
        elif self.type_id.id == self.env.ref('medical_benefit.mbdt_pc').id:
            self.validator_id = self.env.ref('medical_benefit.mbdv_pc')

    @api.one
    def _compute_short_desc(self):
        base = ''
        desc = self.description
        if desc.__len__() > 50:
            base_split = desc.split(',')
            for part in base_split:
                base_split.remove(part)
                base += part
                if base.__len__() > 50:
                    break
            if base_split:
                base += '...'
            self.short_desc = base
        else:
            self.short_desc = desc

    @api.multi
    def name_get(self):
        res = []
        for item in self:
            res.append((item.id, '[%s] %s' % (item.name, item.short_desc)))
        return res

    @api.model
    def _name_search(self, name, args=None, operator='ilike', limit=100, name_get_uid=None):
        if not args and name:
            args = [
                '|',
                ['name', '=ilike', name + '%'],
                ['description', 'ilike', name]
            ]
            operator = 'ilike'
            name = ''
        res = super()._name_search(name=name, args=args, operator=operator, limit=limit, name_get_uid=name_get_uid)
        return res


class MedicalBenefitTariff(models.Model):
    _name = 'medical.benefit.tariff'
    _description = 'Medical Benefit Tariff'

    insurance_plan_id = fields.Many2one(
        'health.insurance.plan', 'Health Insurance', required=True, ondelete='restrict')
    benefit_descriptor_id = fields.Many2one(
        'medical.benefit.descriptor', 'Descriptor', required=True, ondelete='restrict')
    valid_from = fields.Date('Valid From')
    price = fields.Float('price',required=True)

    @api.constrains('valid_from')
    def _check_valid_from_date(self):
        dt = self.valid_from.strftime("%Y-%m-%d")
        tariffs = self.search([
            ('id','!=',self.id),
            ('insurance_plan_id','=',self.insurance_plan_id.id),
            ('benefit_descriptor_id','=',self.benefit_descriptor_id.id),
            ('valid_from','=',dt),
        ])
        if tariffs:
            raise ValidationError(_("There is already another tariff for that insurance and date"))

class HealthInsurancePlan(models.Model):
    _inherit = 'health.insurance.plan'

    tariff_ids = fields.One2many('medical.benefit.tariff', 'insurance_plan_id', string='Tariffs')

class MedicalBenefitEye(models.Model):
    _name = 'medical.benefit.eye'
    _description = "Eye selector for medical benefit"

    name = fields.Selection(
            [
                ('left', 'Left'),
                ('right', 'Right'),
                ('bilateral', 'Bilateral'),
            ],
            string='Name',
    )
    code = fields.Char('Code')

    @api.constrains('code')
    def check_no_duplicated(self):
        mbe = self.env['medical.benefit.eye']
        for eye in self:
            eyes = mbe.search([('code','=',eye.code)])
            if len(eyes) > 1:
                raise ValidationError(_("There is already another eye with code %s" % (eye.code)))

    @api.multi
    def name_get(self):
        res = []
        for eye in self:
            name = dict(eye.fields_get(['name'])['name']['selection'])[eye.name]
            res.append((eye.id,"%s:%s" % (eye.code,_(name))))
        return res

    @api.model
    def _name_search(self, name, args=None, operator='ilike', limit=100, name_get_uid=None):
        if not args and name:
            args = [
                '|',
                ['name', '=ilike', name + '%'],
                ['code', 'ilike', name]
            ]
            operator = 'ilike'
            name = ''
        res = super()._name_search(name=name, args=args, operator=operator, limit=limit, name_get_uid=name_get_uid)
        return res

class MedicalBenefitBudgetLine(models.Model):
    _name = 'medical.benefit.budget.line'
    _description = 'Budget lines for medical benefits'

    name = fields.Char('Name')
    amount = fields.Float('Amount')
    benefit_id = fields.Many2one('medical.benefit','Benefit')

    @api.constrains('amount')
    def _check_amount(self):
        if self.amount < 0:
            raise ValidationError(_("Budget amount must be positive or 0"))


class MedicalBenefit(models.Model):
    _name = 'medical.benefit'
    _description = 'Medical Benefit'


    order_number = fields.Char('Order Number')
    state = fields.Selection(
            [
                ('draft_sofsa', 'Draft SOFSA'),
                ('draft_clinic', 'Draft Clinic'),
                ('invalid', 'Invalid'),
                ('for_invoicing', 'For Invoicing'),
                ('for_reinvoicing', 'For Reinvoicing'),
                ('invoiced', 'Invoiced'),
                ('pending', 'Pending'),
                ('charged', 'Charged'),
                ('not_charged','Not Charged'),
                ('persistent', 'Persistent'),
            ],
            string='Benefit state',
            default=lambda x: x.get_default_state(),
    )
    professional_id = fields.Many2one(
            'res.partner', 'Professional', required=True, ondelete='restrict', default=lambda x: x.get_default_professional())
    patient_id = fields.Many2one(
            'res.partner', 'Patient', required=True, ondelete='restrict')
    insurance_plan_id = fields.Many2one(
            'health.insurance.plan', 'Health Insurance', ondelete='restrict', 
            default=lambda x: x.get_default_insurance_plan_id(),
            domain=lambda x: x.get_domain_insurance_plan_id())
    descriptor_id = fields.Many2one(
            'medical.benefit.descriptor', 'Descriptor', required=True, ondelete='restrict')
    benefit_date = fields.Date('Benefit Date', required=True)
    presentation_date = fields.Date('Presentation date',related='lot_number_id.presentation_date')
    eye = fields.Many2one('medical.benefit.eye','Eye')
    requires_eye = fields.Boolean(string='Requires eye specification', related='descriptor_id.requires_eye')
    user_id = fields.Many2one('res.users', default=lambda x: x.get_user_id(), compute='_check_current_user')
    partner_type = fields.Char('Partner type', compute='_check_partner_type')
    error_notes = fields.Text(string="Reason of invalidity")
    lens_code = fields.Char('Lens Code')
    show_lens = fields.Boolean('Show Lens',compute='_compute_show_lens')
    insurance_ref = fields.Char('Insurance code', related='insurance_plan_id.ref')
    user_insurance_ref = fields.Char(compute='_get_user_insurance_ref', search='_search_user_insurance_ref')
    related_benefit_id = fields.Many2one('medical.benefit','Related benefit')
    budget_line_ids = fields.One2many('medical.benefit.budget.line','benefit_id','Budget lines',ondelete='cascade')
    lot_number_id = fields.Many2one('lot.number','Lot Number', default=lambda x: x.get_lot_number_id())
    price = fields.Float('Price',compute='_get_benefit_price',search='_search_price')
    validator_id = fields.Many2one('medical.benefit.descriptor.validator',string='Validator',related='descriptor_id.validator_id')
    benefit_validation_ids = fields.Many2many('medical.benefit','medical_benefit_related_validation','benefit_original_id','benefit_related_id','Related Benefits')
    benefit_lens_id = fields.Many2one('medical.benefit','Benefit using the lens')
    use_next_lot_number = fields.Boolean('Use next lot number')
    quantity = fields.Float('Quantity',default=1)
    lot_benefits_qty = fields.Integer('Lot Benefits', compute='_compute_lot_benefits_qty')


    @api.constrains('benefit_date')
    def check_benefit_date(self):
        bdate = self.benefit_date
        if bdate:
            if bdate.isoweekday() == 7:
                raise ValidationError(_('Benefit date can not be Sunday'))
            if bdate > datetime.datetime.now().date():
                raise ValidationError(_('Benefit date can not be greater than current date'))
    
    @api.multi
    def _compute_lot_benefits_qty(self):
        benefits = self.env['medical.benefit'].search([('lot_number_id','=',self.lot_number_id.id)])
        self.lot_benefits_qty = len(benefits)

    @api.constrains('quantity')
    def _check_qty(self):
        if self.quantity <= 0:
            raise ValidationError(_("Quantity must be greater than 0"))

    @api.onchange('quantity')
    def onchange_quantity(self):
        self._check_qty()

    def _search_price(self,operator,value):
        benefits = self.env['medical.benefit'].search([('price',operator,value)], limit=None)
        return [('id','in',benefits.ids)]


    @api.depends('insurance_plan_id','descriptor_id','benefit_date','quantity')
    def _get_benefit_price(self):
        tariff = self.env['medical.benefit.tariff']
        for benefit in self:
            price = 0
            descriptor = benefit.descriptor_id
            insurance = benefit.insurance_plan_id
            benefit_date = benefit.benefit_date
            if descriptor and insurance and benefit_date:
                bd = benefit.benefit_date.strftime("%Y-%m-%d")
                tariffs = tariff.search([
                    ('valid_from','<=',bd),
                    ('benefit_descriptor_id','=',descriptor.id),
                    ('insurance_plan_id','=',insurance.id),
                ]) 
                max_tariff = False
                for tr in tariffs:
                    if tr == tariffs[0]:
                        max_tariff = tr
                    elif tr.valid_from > max_tariff.valid_from:
                        max_tariff = tr
                if max_tariff:
                    price = max_tariff.price * benefit.quantity
            benefit.price = price

    @api.multi
    def button_next_lot_number(self):
        lot_number = self.create_lot_number()
        self.lot_number_id = lot_number
        self.env.user_default_lot_number = lot_number
        self.env.user_default_insurance_plan_id = self.insurance_plan_id

    @api.multi
    def get_lot_number_id(self):
        user_ln = self.env.user.default_lot_number
        lot_number = user_ln if user_ln else self.insurance_plan_id.last_lot_number_id
        #if not lot_number:
        #    lot_number = self.create_lot_number()
        return lot_number

    @api.multi
    def create_lot_number(self):
        ln = self.env['lot.number']
        value = self.env['ir.sequence'].next_by_code('medical.benefit.lot.number')
        lot_number = ln.create({
            'insurance_plan_id': self.insurance_plan_id.id,
            'value': value,
        })
        self.lot_number_id = lot_number
        self.insurance_plan_id.last_lot_number_id = lot_number
        return lot_number

    @api.constrains('eye')
    def check_descriptor_eye(self):
        name_os_list = ['I01','I02','I03','I04','I05','I06','I07','I08','I09','I10','I11','I12','I13','I14']
        if self.related_benefit_id and self.descriptor_id:
            if self.descriptor_id.name_os in name_os_list:
                raise ValidationError(_("That descriptor cannot be applied in both eyes (bilateral)"))

    @api.onchange('order_number')
    def onchange_order_number(self):
        if self.order_number:
            benefits = self.env['medical.benefit'].search([('order_number','=',self.order_number)])
            res = {'warning': {'title': _('Warning'),'message': _('A benefit with that order number already exists')}}
            if self.related_benefit_id:
                if len(benefits) > 1:
                    return res 
            elif len(benefits) > 0:
                return res

    @api.constrains('order_number')
    def check_related_order_number(self):
        if self.related_benefit_id:
            if self.related_benefit_id.order_number != self.order_number:
                self.related_benefit_id.order_number = self.order_number

    @api.multi
    def _search_user_insurance_ref(self, operator, value):
        dip = self.env.user.default_insurance_plan_id
        if dip:
            value = dip.ref
            benefits = self.search([('insurance_ref',operator,value)], limit=None)
        else:
            benefits = self.search([], limit=None)
        return [('id','in',benefits.ids)]

    @api.multi
    def _get_user_insurance_ref(self):
        for benefit in self:
            benefit.user_insurance_ref = self.env.user.default_insurance_plan_id.insurance_id.ref

    @api.multi
    def button_validate(self):
        self.validate_lens_code()
        self.validate_benefit()

    @api.multi
    def get_default_state(self):
        user = self.env.user
        partner_type = user.partner_id.partner_type
        if partner_type == 'sofsa_user':
            return 'draft_sofsa'
        elif partner_type == 'clinic' or partner_type == 'professional':
            return 'draft_clinic'

    @api.multi
    def get_domain_insurance_plan_id(self):
        insurance_plan_ids = []
        partner = self.env.user.partner_id
        if partner.partner_type == 'professional':
            for grouper in partner.attended_insurance_ids:
                insurance_plan_ids.extend(self.env['health.insurance.plan'].search([('insurance_id','=',grouper.id)]).ids)
            return [('id','in',insurance_plan_ids)]

    @api.multi
    def _compute_show_lens(self):
        name_os_list = ['I01','I02','I03','I04','I05','I06','I07','I08','I09','I10','I11','I12','I13','I14']
        if self.descriptor_id:
            if self.descriptor_id.name_os in name_os_list:
                self.show_lens = True
            else:
                self.show_lens = False
        else:
            self.show_lens = False

    @api.onchange('descriptor_id')
    def onchange_descriptor_id(self):
        self._compute_show_lens()

    @api.onchange('insurance_plan_id')
    def onchange_insurance_plan_id(self):
        rp = self.env['res.partner']
        partner_ids = []
        professional_ids = []
        res = {
            'domain': {
                'patient_id': [('id','in',[])],
                'professional_id': [('id','in',[])],
                'lot_number_id': [('insurance_plan_id','in',[])]
            },
        }
        if self.insurance_plan_id:
            res['domain'] = {}
            query = ""
            param_id = []
            if self.insurance_plan_id.insurance_id.id == self.env.ref('health_insurance.res_partner_pami').id:
                query = """
                SELECT DISTINCT ON(rp.id) rp.id
                FROM res_partner rp JOIN
                res_partner_insurance rpi ON rpi.patient_id = rp.id JOIN
                health_insurance_plan hip ON rpi.insurance_plan_id = hip.id
                WHERE hip.insurance_id = %s """
                param_id.append(self.env.ref('health_insurance.res_partner_pami').id)
            else:
                query = """
                SELECT DISTINCT ON(rp.id) rp.id
                FROM res_partner rp JOIN
                res_partner_insurance rpi ON rpi.patient_id = rp.id JOIN
                health_insurance_plan hip ON rpi.insurance_plan_id = hip.id
                WHERE rpi.insurance_plan_id = %s """
                param_id.append(self.insurance_plan_id.id)
            self.env.cr.execute(query,param_id)
            partner_ids = []
            for l in self.env.cr.dictfetchall():
                partner_ids.append(l['id'])
            res['domain']['patient_id'] = [('id','in',partner_ids)]
            for professional in rp.search([('partner_type','=','professional')]):
                if self.insurance_plan_id.insurance_id.id in professional.attended_insurance_ids.ids:
                    professional_ids.append(professional.id)
            res['domain']['professional_id'] = [('id','in',professional_ids)]
            if self.patient_id:
                for insurance in self.patient_id.health_insurance_ids:
                    if insurance.insurance_plan_id.id != self.insurance_plan_id.id:
                        self.patient_id = ""
            if self.professional_id:
                if self.insurance_plan_id.insurance_id.id not in self.professional_id.attended_insurance_ids.ids:
                    self.professional_id = ""
            partner_pami = self.env.ref('health_insurance.res_partner_pami')
            if self.insurance_plan_id.insurance_id == partner_pami:
                insurances = self.env['health.insurance.plan'].search([('insurance_id','=',partner_pami.id)])
                res['domain']['lot_number_id'] = [('insurance_plan_id','in',insurances.ids)]
            else:
                res['domain']['lot_number_id'] = [('insurance_plan_id','=',self.insurance_plan_id.id)]
            if not self.lot_number_id or self.lot_number_id.insurance_plan_id != self.insurance_plan_id:
                self.lot_number_id = self.insurance_plan_id.last_lot_number_id
            if self.insurance_plan_id.id != self.env.user.default_insurance_plan_id.id:
                self.use_next_lot_number = True
            else:
                self.use_next_lot_number = False
                self.lot_number_id = self.env.user.default_lot_number
        else:
            self.patient_id = ""
            self.professional_id = ""
            self.lot_number_id = ""
        return res
    
    @api.multi
    def validate_lens_code(self):
        if self.lens_code:
            benefit = self.search([('lens_code','=',self.lens_code),('id','!=',self.id)])
            if len(benefit) > 0:
                self.write({
                        'error_notes': _("The lens code is already in use"),
                        'state': 'invalid',
                        'validation': True,
                        'benefit_lens_id': benefit[0].id,
                })

    @api.multi
    def write(self, vals):
        if len(self) > 1:
            for benefit in self:
                super(MedicalBenefit, benefit).write(vals)
                #benefit.validate_order_number()
                benefit.validate_lens_code()
                benefit.add_insurance_patient()
                if benefit.state == 'for_invoicing':
                    benefit.validate_benefit()
        elif len(self) == 1:
            if 'validation' not in vals:
                if 'lot_number' in vals:
                    if vals['lot_number'] != "":
                        self.env.user.default_lot_number = self.env['lot.number'].browse(vals['lot_number'])
                        self.env.user.default_insurance_plan_id = self.insurance_plan_id
                super(MedicalBenefit, self).write(vals)
                #self.validate_order_number()
                self.validate_lens_code()
                self.add_insurance_patient()
                if self.state == 'for_invoicing':
                    self.validate_benefit()
            else:
                del vals['validation']
                super(MedicalBenefit, self).write(vals)
        else:
            super(MedicalBenefit, self).write(vals)

    @api.model
    def create(self, vals):
        importing = False
        use_next_ln = False
        if 'importing' in vals:
            importing = True
            del vals['importing']
        if 'use_next_lot_number' in vals:
            if vals['use_next_lot_number'] == True:
                use_next_ln = True
                vals['use_next_lot_number'] = False
        if 'eye' in vals and vals['eye'] == self.env.ref('medical_benefit.bilateral_eye').id:
            vals['eye'] = self.env.ref('medical_benefit.left_eye').id
            benef_left = super(MedicalBenefit, self).create(vals)
            if use_next_ln:
                benef_left.create_lot_number()
                vals['lot_number_id'] = benef_left.lot_number_id.id
            #benef_left.validate_order_number()
            vals['eye'] = self.env.ref('medical_benefit.right_eye').id
            vals['related_benefit_id'] = benef_left.id
            benef_right = self.create(vals)
            benef_left.related_benefit_id = benef_right
            if not importing:
                benef_left.validate_benefit()
                self.env.user.default_lot_number = benef_left.lot_number_id
                self.env.user.default_insurance_plan_id = benef_left.insurance_plan_id
            return benef_left
        benef = super(MedicalBenefit, self).create(vals)
        #benef.validate_order_number()
        benef.validate_lens_code()
        benef.add_insurance_patient()
        if not importing:
            if use_next_ln:
                benef.create_lot_number()
            benef.validate_benefit()
            self.env.user.default_lot_number = benef.lot_number_id
            self.env.user.default_insurance_plan_id = benef.insurance_plan_id
        return benef

    @api.multi
    def add_insurance_patient(self):
        add_insurance = True
        for insurance in self.patient_id.health_insurance_ids:
            if self.insurance_plan_id.insurance_id.id == self.env.ref('health_insurance.res_partner_pami').id:
                if insurance.insurance_plan_id.insurance_id.id == self.env.ref('health_insurance.res_partner_pami').id:
                    add_insurance = False
                    break
            else:
                if insurance.insurance_plan_id.id == self.insurance_plan_id.id:
                    add_insurance = False
                    break
        if add_insurance:
            if self.insurance_plan_id.insurance_id.id == self.env.ref('health_insurance.res_partner_pami').id:
                self.patient_id.health_insurance_ids = [(0,0,{'insurance_plan_id': self.insurance_plan_id.insurance_id.main_insurance_plan_id.id})] 
            else:
                self.patient_id.health_insurance_ids = [(0,0,{'insurance_plan_id': self.insurance_plan_id.id})] 

    @api.multi
    def validate_order_number(self): #Not used anymore
        if self.order_number:
            related_id = self.related_benefit_id.id if self.related_benefit_id else False
            benefits = self.search_count([('order_number','=',self.order_number),('id','!=',self.id),('id','!=',related_id)])
            if benefits >= 1:
               self.write({
                    'error_notes': _("That order number is already in use"),
                    'state': 'invalid',
                    'validation': True,
                    'benefit_validation_ids': [(6,0,benefits.ids)],
                })

    @api.multi
    def validate_benefit(self):
        validator_id = self.descriptor_id.validator_id
        benefit_date = self.benefit_date
        eye = self.eye
        descriptor_id = self.descriptor_id
        patient_id = self.patient_id
        benefit_model = self.env['medical.benefit']
        if validator_id and benefit_date and eye:
            period = int(validator_id.period)
            control = validator_id.control
            search_list = [
                    ('descriptor_id','=',descriptor_id.id),
                    ('patient_id','=',patient_id.id),
                    ('state','in',['for_invoicing','persistent']),
                    ('id','!=',self.id)]
            if period > 0:
                min_date = benefit_date - datetime.timedelta(days=period)
                search_list.append(('benefit_date','>=',min_date))
            if control == 'per_eye':
                search_list.append(('eye','in',[eye.id]))
            benefits = benefit_model.search(search_list)
            benefits += self
            sum_benefits = 0
            for benefit in benefits:
                sum_benefits += benefit.quantity
                if sum_benefits > validator_id.quantity:
                    vq = validator_id.quantity
                    dn = descriptor_id.name
                    days = validator_id.period
                    control_string = dict(validator_id._fields['control'].selection).get(validator_id.control)
                    if int(days) > 0:
                        error_notes = _("You have more than %d benefits with descriptor %s in the last %s days for this patient.\n") % (vq,dn,days)
                    else:
                        error_notes = _("You have more than %d benefits with descriptor %s in total for this patient.\n") % (vq,dn)
                    self.write({
                        'error_notes': error_notes,
                        'state': 'invalid',
                        'validation': True,
                        'benefit_validation_ids': [(6,0,benefits.ids)],
                    })
                    return 0
        if self.descriptor_id.requires_eye:
            if not eye:
                error_notes = "Eye field cannot be empty for this descriptor"
                self.write({
                    'error_notes': error_notes,
                    'state': 'invalid',
                    'validation': True,
                    'benefit_validation_ids': [(6,0,[])],
                })
                return 0
        self.write({
            'error_notes': "",
            'state': 'for_invoicing',
            'validation': True,
            'benefit_validation_ids': [(6,0,[])],
        })
        return 1


    @api.multi
    def name_get(self):
        res = []
        for benefit in self:
            if benefit.order_number:
                name = benefit.order_number + " - "
            else:
                name = ""
            name += benefit.patient_id.name + " - " + benefit.descriptor_id.name_os
            res.append((benefit.id,name))
        return res

    @api.multi
    def get_user_id(self): #For creation
        return self.env.user

    @api.multi
    def get_default_insurance_plan_id(self):
        user = self.env.user
        return user.default_insurance_plan_id


    @api.multi
    def get_default_professional(self):
        partner = self.env.user.partner_id
        return partner if partner.partner_type == 'professional' else ''


    @api.multi
    def _check_current_user(self): #Keep the user updated when editing
        for benefit in self:
            benefit.user_id = self.env.user

    @api.depends('user_id')
    @api.multi
    def _check_partner_type(self):
        for benefit in self:
            pt = self.env.user.partner_id.partner_type
            benefit.partner_type = pt

    @api.model
    def cron_reset_lot_number_sequence(self):
        sequence = self.env.ref('medical_benefit.seq_medical_benefit_lot_number')
        sequence.write({'number_next_actual': 1})

    @api.model
    def group_report_values(self):
        medical_benefit = self.env['medical.benefit']
        benefits = medical_benefit.search(['state','=','for_invoicing'])
        benefits = benefits.sorted(key=lambda x: x.insurance_id.ref)
        return benefits

    @api.multi
    def action_change_lot_date(self):
         return {
            'context': self.env.context,
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'lot.number',
            'res_id': self.lot_number_id.id,
            'view_id': self.env.ref('medical_benefit.view_date_lot_number_form').id,
            'type': 'ir.actions.act_window',
            'target': 'new',
        }

    @api.multi
    def action_charge_related_benefit(self):
        return {
            'context': self.env.context,
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'medical.benefit',
            'res_id': self.related_benefit_id.id,
            'view_id': self.env.ref('medical_benefit.view_medical_benefit_form').id,
            'type': 'ir.actions.act_window',
            'target': 'main',
        }


    @api.multi
    def action_charge_lot_benefits(self):
        context = dict()
        context.update(self.env.context)
        context.update({'create': False, 'order_by': ['create_date'], 'search_default_insurance_code_filter': False})
        action = self.env.ref('medical_benefit.medical_benefit_action').read()[0]
        action['domain'] = str([('lot_number_id', '=', self.lot_number_id.id)])
        action['context'] = context
        action['target'] = 'main'
        return action

    @api.multi
    def button_set_uncollected(self):
        self.write({
            'state': 'not_charged',
            'validation': True,
        })
        medical_records = self.env['medical.record'].search([('benefit_id','=',self.id)])
        medical_records.write({
            'description': _('Uncollected benefit'),
        })
