# -*- coding: utf-8 -*-
###############################################################################
#   Copyright (c) 2020 Eynes / E-Mips (Marco Folco)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
###############################################################################

import logging
import base64
import csv
import io
import resource
from datetime import datetime

from odoo import models, api, fields, _, exceptions
from odoo.tools import config
# from odoo.exceptions import except_orm
# from odoo.addons.decimal_precision import decimal_precision as dp
# from odoo.tools import DEFAULT_SERVER_DATE_FORMAT, \
#         DEFAULT_SERVER_DATETIME_FORMAT, float_compare


_logger = logging.getLogger(__name__)

class Tariff(models.TransientModel):
    _name = 'tariff'
    _description = 'Medical Benefit Tariff Import'

    error_list = fields.Char('Errors')
    benefit_descriptor_id = fields.Char('Descriptor')
    price = fields.Char('Price')

    wizard_import_id = fields.Many2one('wizard.tariff.import')

class WizardImportTariff(models.TransientModel):
    _name = 'wizard.tariff.import'
    _description = 'Wizard for import tariffs'

    state = fields.Selection(
            [
                ('init', 'Init'),
                ('done', 'Done'),
                ('reimport', 'Reimport'),
            ],
            string="State",
            default=lambda wiz: wiz.get_default_state(),
    )

    filename = fields.Char('Filename')

    tariff_file = fields.Binary(string='Tariff file', filename='filename')                                                                                                           
    insurance_plan_id = fields.Many2one('health.insurance.plan', string='Insurance')

    total_imported = fields.Integer(string="Total records imported",
            default=lambda wiz: wiz.get_total_imported())
    total_created = fields.Integer(string="Total tariffs created")
    total_not_imported = fields.Integer(string="Total records not imported")
    total_existent = fields.Integer(string="Total tariffs existent")

    not_imported = fields.One2many('tariff', 'wizard_import_id', string="Records to import")

    valid_from = fields.Date('Valid From', required=True)
    
    override_tariffs = fields.Boolean('Override existent tariffs')


    def get_default_state(self):
        state="init"
        if self.total_imported > 0 or self.total_not_imported > 0:
            state = "done"
        
        return state

    def get_total_imported(self):
        try:
            return len(self.total_imported())
        except TypeError:
            return 0

    @api.multi
    def button_generate_transient(self):
        transients = self.env.context.get('transients')
        if len(transients) > 0:
            transient_ids = []
            for values in transients.values():
                transient = self.create_transient(values)
                transient_ids.append(transient.id)
            self.write({
                'state': 'reimport',
            })
            return {
                'context': self.env.context,
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'wizard.tariff.import',
                'res_id': self.id,
                'view_id': False,
                'type': 'ir.actions.act_window',
                'target': 'new',
            }
        return {
            'type': 'ir.actions.do_nothing',
        }

    @api.multi
    def delete_headers(self):
        self.file_reader = self.file_reader[1:]

    @api.multi
    def button_import(self):
        file_decoded = base64.b64decode(self.tariff_file)
        try:
            decode_type = 'ISO-8859-1'
            data = io.StringIO(file_decoded.decode(decode_type))
        except Exception:
            decode_type = 'utf-8'
            data = io.StringIO(file_decoded.decode(decode_type))
        data.seek(0)
        self.file_reader = []
        csv_reader = csv.reader(data, delimiter=';')
        self.file_reader.extend(csv_reader)
        self.delete_headers()
        self.row_num = len(self.file_reader)
        if self.row_num > 0:
            self.col_num = len(self.file_reader[0])
            self.import_tariff()   
            self = self.with_context(transients=self.transients)
            self.not_imported = ""
            return {
                'context': self.env.context,
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'wizard.tariff.import',
                'res_id': self.id,
                'view_id': False,
                'type': 'ir.actions.act_window',
                'target': 'new',
            }
        else:
            _logger.info('Trying to import an empty file! Aborted.')

    def norm_descriptor_code(self, value):
        descriptor = self.env['medical.benefit.descriptor'].search([('name_os','=',value)])
        if descriptor:
            return descriptor.id
        else:
            raise AttributeError(_('No descriptor found with that code'))

    def norm_descriptor_price(self, value):
        value = value.strip()
        price = value
        try:
            float(price)
        except Exception:
            raise AttributeError(_('Invalid price format'))

        return price

    def build_field_column_map(self):
        self.field_column_map = {
                0: {"fieldname": "benefit_descriptor_id", "function": self.norm_descriptor_code},
                1: {"fieldname": "price", "function": self.norm_descriptor_price},
        }

    def create_transient(self, values):
        values['wizard_import_id'] = self.id
        transient = self.env['tariff'].create(values)
        return transient

    def _prepare_data(self, values):
        data = {
                'write_date': fields.Datetime.now(),
        }

        do_import = True

        transient_values = {
                'error_list': "",
        }
        error_count = 1

        for col_pos, field_function in self.field_column_map.items():
            try:
                value = values[col_pos]
                transient_values[field_function['fieldname']] = value
                if field_function['fieldname']: #If fieldname is empty, it means we only want to execute a check
                    try:
                        data[field_function['fieldname']] = field_function['function'](value)
                    except AttributeError as e:
                        do_import = False
                        transient_values['error_list'] += "(%s)%s  " % (error_count,str(e))
                        error_count += 1
                else:
                    field_function['function'](value)
            except KeyError:
                continue

        if not do_import:
            data = False

        return data,transient_values

    def get_values_list(self, row_number):
        values = []
        fr = self.file_reader[row_number]
        if fr:
            values.extend([fr[0],fr[3]])
            return values
        return False

    def get_records(self):
        return self.not_imported

    def process_file(self):
        self.build_field_column_map()
        if self.state == 'init':
            for row_number in range(self.row_num):
                values = self.get_values_list(row_number)
                if values:
                    data, transient_values = self._prepare_data(values)
                    yield data, transient_values
        elif self.state == 'reimport':
            records = self.get_records()
            for transient in records:
                values_trans = transient.read()[0]
                values = []
                for v in values_trans.items():
                    if isinstance(v[1], str) and v[0] != 'error_list':
                        values.append(v[1])
                data, transient_values = self._prepare_data(values)
                yield data, transient_values

    def create_tariff(self, data):
        mbt = self.env['medical.benefit.tariff']
        hip = self.env['health.insurance.plan']

        if None in data.values():
            return 0

        created = 0
        existent = 0
        insurance = self.insurance_plan_id.insurance_id
        ref_pami = self.env.ref('health_insurance.res_partner_pami')
        plans = hip.search([('insurance_id','=',self.insurance_plan_id.insurance_id.id)]) if insurance.id == ref_pami.id else self.insurance_plan_id
        for plan in plans:
            data['insurance_plan_id'] = plan.id
            tariff = mbt.search([
                ('insurance_plan_id','=',data['insurance_plan_id']),
                ('valid_from','=',self.valid_from),
                ('benefit_descriptor_id','=',data['benefit_descriptor_id'])
            ])
            if not tariff:
                tariff = mbt.create(data)
                tariff.valid_from = self.valid_from
                created += 1
            else:
                if self.override_tariffs:
                    tariff.write({'price': data['price']})
                existent += 1
        return created, existent

    def clean_transients(self):
        self.not_imported = ""

    def import_tariff(self):
        prev = config['limit_time_real']
        config['limit_time_real'] = False
        transient_ids = []
        self.transients = {}
        total = 0
        existent = 0
        self.total_created = 0
        not_imported = 0
        self.last_transient = 0
        for data, transient_values in self.process_file():
            if data:
                cr, ex = self.create_tariff(data)
                total += cr
                existent += ex
                if total % 50 == 0:
                    _logger.info('{} tariffs in queue!'.format(total))
            else:
                if transient_values['benefit_descriptor_id'] != "" or transient_values['price'] != "":
                    not_imported += 1
                    self.transients[self.last_transient] = transient_values
                    self.last_transient += 1

        self.write({
            'total_imported': total,
            'total_existent': existent,
            'total_not_imported': not_imported,
            'state': 'done',
        })
        self.clean_transients()
        config['limit_time_real'] = prev
